/***************************************************************************
 *   Copyright (C) by GFZ Potsdam                                          *
 *                                                                         *
 *   You can redistribute and/or modify this program under the             *
 *   terms of the SeisComP Public License.                                 *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   SeisComP Public License for more details.                             *
 ***************************************************************************/

#include <seiscomp3/gui/map/layer.h>

#include <seiscomp3/core/interfacefactory.ipp>

#include <QContextMenuEvent>

IMPLEMENT_INTERFACE_FACTORY(Seiscomp::Gui::Map::Layer, SC_GUI_API);

namespace Seiscomp {
namespace Gui {
namespace Map {

IMPLEMENT_ROOT_RTTI(Layer, "Map::Layer")
IMPLEMENT_RTTI_METHODS(Layer)

Layer::Layer(QObject* parent) : QObject(parent), Core::BaseObject(),
    _visible(true), _antiAliasing(false) {
}

Layer::~Layer() {
}

void Layer::calculateMapPosition(const Map::Canvas *canvas) {
}

bool Layer::filterContextMenuEvent(QContextMenuEvent* e, QWidget*) {
	return false;
}

void Layer::setDescription(const QString& d) {
	_description = d;
}

const QString& Layer::description() const {
	return _description;
}

void Layer::init(const Seiscomp::Config::Config&) {
}

void Layer::setName(const QString& n) {
	_name = n;
}

const QString& Layer::name() const {
	return _name;
}

void Layer::setVisible(bool v) {
	_visible = v;
	foreach ( Seiscomp::Gui::Map::Legend* legend, _legends ) {
		legend->setVisible(v);
	}
}

bool Layer::isVisible() const {
	return _visible;
}

void Layer::setAntiAliasingEnabled(bool v) {
	_antiAliasing = v;
}

bool Layer::isAntiAliasingEnabled() const {
	return _antiAliasing;
}

Legend* Layer::legend(int i) const {
	if ( i < 0 || i >= _legends.size() ) return NULL;

	return _legends[i];
}

QMenu* Layer::menu(QWidget*) const {
	return NULL;
}

Layer& Layer::operator =(const Layer &other) {
	_name = other._name;
	_description = other._description;
	_visible = other._visible;
	_antiAliasing = other._antiAliasing;

	_legends.clear();
	foreach ( Legend* legend, other._legends ) {
		Legend *tmp = legend->clone();
		if ( tmp != NULL ) _legends.append(tmp);
	}

	return *this;
}

bool Layer::addLegend(Seiscomp::Gui::Map::Legend *legend) {
	if ( _legends.contains(legend) ) return false;

	connect(legend, SIGNAL(destroyed(QObject*)),
	        this, SLOT(onObjectDestroyed(QObject*)));

	_legends.append(legend);
	return true;
}

bool Layer::removeLegend(Seiscomp::Gui::Map::Legend *legend) {
	if ( !_legends.contains(legend) ) return false;

	disconnect(legend, SIGNAL(destroyed(QObject*)),
	           this, SLOT(onObjectDestroyed(QObject*)));

	_legends.removeAll(legend);
	return true;
}

void Layer::onObjectDestroyed(QObject *object) {
	_legends.removeAll(static_cast<Seiscomp::Gui::Map::Legend*>(object));
}

} // namespace Map
} // namespce Gui
} // namespace Seiscomp
